@@ -70,10 +70,13 @@ namespace galera
                                                   bool            create) = 0;
         virtual void discard_local_conn_trx(wsrep_conn_id_t conn_id) = 0;
 
-        virtual wsrep_status_t replicate(TrxHandleMaster&   trx,
-                                         wsrep_trx_meta_t*  meta) = 0;
-        virtual wsrep_status_t certify(TrxHandleMaster&     trx,
-                                       wsrep_trx_meta_t*    meta) = 0;
+        virtual wsrep_status_t replicate(TrxHandleMaster& trx,
+                                         wsrep_trx_meta_t* meta,
+                                         const wsrep_seq_cb_t* seq_cb)
+            = 0;
+        virtual wsrep_status_t certify(TrxHandleMaster& trx,
+                                       wsrep_trx_meta_t* meta)
+            = 0;
         virtual wsrep_status_t replay_trx(TrxHandleMaster&  trx,
                                           TrxHandleLock&    lock,
                                           void*             replay_ctx) = 0;
