@@ -214,6 +214,11 @@ BOOLEAN _gu_no_db_    = TRUE;	         /* TRUE if no debugging at all      */
  */
 
 
+/*
+ * Galera does not provided _sanity which is used when SAFEMALLOC is
+ * defined
+ */
+#undef SAFEMALLOC
 
 IMPORT int _sanity(const char *file, uint line);
 
@@ -351,7 +356,7 @@ static char *static_strtok(char *s1, char chr);
 
 #undef EXISTS
 #if !defined(HAVE_ACCESS) || defined(MSDOS)
-#define EXISTS(pathname) (FALSE)			   /* Assume no existance */
+#define EXISTS(pathname) (FALSE)			   /* Assume no existence */
 #define Writable(name) (TRUE)
 #else
 #define EXISTS(pathname)	 (access (pathname, F_OK) == 0)
@@ -706,6 +711,7 @@ _gu_db_push_(const char *control)
 	case 'A':
 	case 'O':
 	    _gu_db_stack->flags |= FLUSH_ON_WRITE;
+            // fall through
 	case 'a':
 	case 'o':
 	    if (*scan++ == ',') {
