@@ -42,7 +42,7 @@ namespace gu
               buf_size_ (buf_size),
               reserve_  (reserve)
         {
-            assert(buf_size >  0);
+            assert(buf_size_ >  0);
             assert(reserve   >= 0);
             pool_.reserve(reserve_);
         }
