@@ -1,8 +1,9 @@
-// Copyright (C) 2009 Codership Oy <info@codership.com>
+// Copyright (C) 2009-2017 Codership Oy <info@codership.com>
 
 #include <cstdlib>
 #include <cstdio>
 #include <string>
+#include <unistd.h>
 
 extern "C" {
 #include "../src/gu_conf.h"
@@ -40,5 +41,7 @@ int main(int argc, char* argv[])
     if (log_file != 0) fclose(log_file);
     printf ("Total tests failed: %d\n", failed);
 
+    if (0 == failed && 0 != log_file) ::unlink(LOG_FILE);
+
     return failed == 0 ? EXIT_SUCCESS : EXIT_FAILURE;
 }
