@@ -11,6 +11,8 @@ struct Suite;
 Suite* types_suite();
 /* Tests for utilities */
 Suite* util_suite();
+/* Util nondeterministic tests */
+Suite* util_nondet_suite();
 /* Tests for logger */
 Suite* logger_suite();
 /* Tests for message buffer implementations */
@@ -23,6 +25,8 @@ Suite* concurrent_suite();
 Suite* tcp_suite();
 /* Tests for GMcast transport */
 Suite* gmcast_suite();
+/* Tests for GMcast nondeterministic transport */
+Suite* gmcast_nondet_suite();
 /* Tests for EVS transport */
 Suite* evs_suite();
 /* Better evs suite */
@@ -31,5 +35,9 @@ Suite* evs2_suite();
 Suite* vs_suite();
 /* Tests for PC transport */
 Suite* pc_suite();
+/* PC nondeterministic tests */
+Suite* pc_nondet_suite();
+/* Fair send queue suite */
+Suite* fair_send_queue_suite();
 
 #endif // CHECK_GCOMM_HPP
