@@ -6,6 +6,7 @@
 
 #include <map>
 #include <stdexcept>
+#include <iostream> // std::cerr
 
 static gu::Config conf;
 
@@ -87,7 +88,6 @@ public:
 
     ~Server()
     {
-        delete listener_;
         pnet_.erase(&pstack_);
         pstack_.pop_proto(this);
     }
@@ -144,7 +144,7 @@ private:
     gu::URI                           uri_;
     gcomm::Protonet&                  pnet_;
     gcomm::Protostack                 pstack_;
-    gcomm::Acceptor*                  listener_;
+    std::shared_ptr<gcomm::Acceptor>  listener_;
     std::map<const void*, gcomm::SocketPtr> smap_;
     const std::string                 msg_;
 };
